# Alexandr Lambov, © 2020

# -*- coding: utf-8 -*-
import csv
import matplotlib.pyplot as plt
import scipy.stats as stats

from statistics import mean 

x_axis_text = "Moldovans + Romainians in the district (2014 census), %"

def val_x(row):
    return float(row['percent_romanians']) + float(row['percent_moldovans'])

def val_y(row):
    votes_sandu = float(row['votes_sandu'])
    votes_dodon = float(row['votes_dodon'])
    
    return votes_sandu/(votes_sandu + votes_dodon)*100

def best_fit(x, y):
    mean_x = mean(x)
    mean_y = mean(y)
    
    num = 0
    denom = 0
    
    for i in range(len(x)):
        num += (x[i] - mean_x)*(y[i] - mean_y)
        denom += (x[i] - mean_x)**2
    
    slope = num/denom
    intercept = mean_y - slope*mean_x
    
    return slope, intercept

def r_word(r):
    abs_r = abs(r)
    
    if abs_r < 0.2:
        return "very weak"
    elif abs_r < 0.5:
        return "weak"
    elif abs_r < 0.7:
        return "medium"
    elif abs_r < 0.9:
        return "strong"
    else:
        return "very strong"
    
def p_word(p):
    if p <= 0.05:
        return 'significant'
    else:
        return 'nonsignificant'
    
def description_text(n, r, p):
    return '\n'.join((
        r"Districts: %d" %n,
        r"(no diaspora or Transnistria)",
        r"Pearson's correlation: %.2f (%s)" % (r, r_word(r)),
        r"p-value: %.10f (%s)" % (p, p_word(p))))

def highlight(rows, district, plt):
    row = next(row for row in rows if row['UAT'] == district)
    plt.annotate(district, (val_x(row), val_y(row)), fontsize=7)
        

# Open the CSV
with open('Elections.csv', newline='') as csvfile:
    reader = csv.DictReader(csvfile, delimiter=';', quotechar='"')
    rows = list(reader)
    rows.pop(0)
    
    # District data
    x = list(map(val_x, rows))
    y = list(map(val_y, rows))
    
    # Best fit line
    slope, intercept = best_fit(x, y)
    best_x = [min(x), max(x)]
    best_y = list(map(lambda x: x*slope + intercept, best_x))
    
    # Statistical indicies
    r, p = stats.pearsonr(x, y)
    
    # Show the data
    plt.title("Presidential Elections RM 15.11.2020")
    
    plt.plot(x, y, 'yo', markersize=2)
    plt.plot(best_x, best_y)
    
    highlight(rows, 'mun. Chisinau', plt)
    highlight(rows, 'mun. Balti', plt)
    highlight(rows, 'UTA Gagauzia', plt)
    
    plt.xlabel(x_axis_text)
    plt.ylabel('Votes for Sandu in the district, %')
    
    # Place a text box in upper left in axes coords
    props = dict(boxstyle='round', facecolor='wheat', alpha=0.2)
    
    plt.text(min(x), max(y), 
             description_text(len(x), r, p), 
             fontsize=9,
             verticalalignment='top', 
             bbox=props)
    
    
    plt.show()
